
N <- c(10,20,50,100,500,1000)

P <- c(0.05,0.1,0.25,0.5)


# Normal approximation to binomial #


require(boot)

norm_approx_binomci <- function(x,n,a = 0.95){

phat <- sum(x) / n

var <- phat * (1 - phat)/ n

norm.ci(boot.out=NULL, conf = a, index=1, var.t0=var,t0=phat )

}




# Bootstrap t and percentile confidence interval #


# B <- No of Bootstrap samples #


bootstrap_t <- function(x,B,n) {   # Function to obtain bootstrap t and #
                                   # percentile confidence interval for given n and p #

t1 <- matrix( 0,B,1 )             # A column matrix of statistics #

z <- matrix( 0,B,1 )              

cv <- rep( 0,2 )                  # Critical values for bootstrap t CI #            

ci <- rep( 0,4 )                  # Bootstrap t & percentile interval limits for given n & p #

t <- sum(x)/ n                      # Value of statistic for parent sample #

for(i in 1: B) {                     # B bootstrap samples corresponding statistic values and z values #

   y  <- sample(x,replace = TRUE)

   t1[i] <- sum(y)/ n 
   
   z[i,] <- ( t1[i] - t )/ sqrt( var(y)/ n )

  
}

   cv <- c( sort(z)[floor(B * 0.025)], sort(z)[floor(B*0.975)] ) 

    V <- var( x )/ n     # Variance of parent sample #

    if ( V > 0 ) 
      {
       
        ci[1]<- t  - cv[2] * sqrt( V )  #lower limit#

        ci[2] <- t - cv[1] * sqrt( V )  #upper limit#
      
      }
     if ( V == 0 )
      {
        ci[1] <-  t 

        ci[2] <-  t 
       
      }

     ci[3:4] <- c( sort( t1 )[floor( B*0.025 )], sort( t1 )[floor( B*0.975 )] )
 
     return (ci)   
     
}




# coverage probability #


coverage_probability <- function(n,p,B1,B){

cp1 <- rep( 0,3 )      # Coverage probability vector for given n and p #

intv1 <- rep( 0,3 )    # Length of the B1 th CI for given n and p #

for( k in 1:B1 )

{

  a <- rep(0,6)

  x <- rbinom( n,1,p )

  a[1:2 ] <- norm_approx_binomci(x,n,a = 0.95)[2:3]

  a[3:6 ] <- bootstrap_t(x,B,n)

  for( m in 1:3 )
     
     intv1[m] <- intv1[m] + a[2*m] - a[(2*m-1)] 

  for( l in 1:3 ){                 # To calculate coverage probablity #
 
     if( a[(2*l-1)]<= p && a[2*l] >= p && l < 4 )

         cp1[l] <- cp1[l] + 1  
  
}
 
}

 
return( list( cp1/B1,intv1/B1 ) )

}



cp <- matrix( 0,24,3 )      # Matrix of coverage probabilities for all n and p#

intvl <- matrix( 0,24,3 )   # Matrix of lengths of B1 th CI for all n and p #

for( i in 1:length(P) )

{
   
  p <- P[i]

  for( j in 1:length(N) )

{

  n <- N[j]

  k <- coverage_probability(n,p,5000,50)

  cp[6*(i-1)+j , ] <- t(as.matrix(unlist( k )[1:3])) # coverage probability #

  intvl[6*(i-1)+j , ] <- t(as.matrix(unlist( k )[4:6]))    # length of CI #


}

}
 

# Here we get coverage proababilty as well as length of the CI for each of the three methods #



# To Plot the intervals in case of Normal approximation to binomial for given n and p #


try <- function( n,p,B,B1 ) {

intv <- matrix( 0,B1,6 )

for( k in 1:B1 )

{

  x <- rbinom( n,1,p )

  intv[k,1:2] <- norm_approx_binomci(x,n,a = 0.95)[2:3] # Normal approximation CI #

  intv[k,3:6 ] <- bootstrap_t(x,B,n)  # Bootstrap t and percentile CI #

}

  contain <- matrix( 0,3,B1 )

  cp1 <- rep( 0,3 )

  for( l in 1:3 ){

     contain[l,] <- numeric(B1)
 
     contain[l,] <- ( intv[,2*l-1]<= p & intv[,2*l] >= p ) 
  
      cp1[l] <- sum( contain[l,] ) / B1     # Coverage probability #
  }

     plot(0,0,type="n",xlim=c(0,1),ylim=c(0,B1),xlab="",ylab="",main="bootstrap conf intervals")
  
     segments(intv[,1],1:B1,intv[,2],1:B1,col=ifelse(contain,"black","red"))

     abline(v=p,col="blue")

  }



# The plot for the other two mwthods can be obtained by doing proper changes in the function try( n,p,B,B1 )

# segments(intv[,3],1:B1,intv[,4],1:B1,col=ifelse(contain,"black","red"))  for bootstrap t CI

# segments(intv[,5],1:B1,intv[,6],1:B1,col=ifelse(contain,"black","red"))  for bootstrap percentile CI





# Only CIs by usng each of the three methods can be obtained as follows #


# To obtain confidence intervals by using normal approximation to binomial #


cl <- matrix(0,24,2)       # Matrix of confidence intervals #

for( i in 1: length(P))
{
   p <- P[i]

   for( j in 1: length(N))
{

       n <- N[j]

       set.seed( 1 )

       x <- rbinom(n,1,p)

       cl[(6*(i-1)+j),1] <- norm_approx_binomci( x,n , 0.95 )[2]  # lower limit#

       cl[(6*(i-1)+j),2] <- norm_approx_binomci( x,n, 0.95 )[3]  # upper limit #

}
       
}

write.table( file = "confidence limits.csv", matrix((cl[,2]-cl[,1]),6,4), sep = ",") # Length of confidence interval #


 


# To obtain bootstrap t and percentile CI for various values of n and p #


interval <- function( N,P,B )
{

cl <- matrix(0,24,4)

for( i in 1:length(P)) 
{  
   p <- P[i]

   for( j in 1:length(N) )
{
   n <- N[j]

   set.seed( 34987 )

   x <- rbinom( n,1,p )

   cl[(6*(i-1)+j), ] <- bootstrap_t (x,B,n) # Bootstrap t and percentile  CI #

}

}

cl

}




data <- interval( N,P,50 )


write.table( file = "Bootstrap t.csv", matrix( (data[,1] - data[,2]),6,4 ) ,sep = "," )#

write.table( file = "percentile.csv", matrix( (data[,4] - data[,3]),6,4 ) ,sep = "," )#






  


  







 





